##############################################################################
#
# Copyright (c) 2004 Chris McDonough, Paul Winkler, Jens Vagelpohl and
# Contributors.  All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os

_urandomfd = None
def urandom(n):
    """urandom(n) -> str

    Return a string of n random bytes suitable for cryptographic use.

    """
    global _urandomfd
    if _urandomfd is None:
        try:
            _urandomfd = os.open("/dev/urandom", os.O_RDONLY)
        except:
            _urandomfd = NotImplementedError
    if _urandomfd is NotImplementedError:
        raise NotImplementedError("/dev/urandom (or equivalent) not found")
    bytes = ""
    while len(bytes) < n:
        bytes += os.read(_urandomfd, n - len(bytes))
    return bytes


def new_uuid():
    return ''.join(['%02x' % ord(x) for x in urandom(16)])

def new_id():
    return ''.join(['%02x' % ord(x) for x in urandom(4)])

def is_uuid(s):
    if len(s) != 32:
        return False
    for char in s:
        if char not in '0123456789abcdef':
            return False
    return True

if __name__ == '__main__':
    import time
    dir = '/tmp/uuidtest'
    start = time.time()
    run = 0
    while 1:
        if run % 1000 == 0:
            print '%s runs finished' % run
        uuid = new_uuid()
        path = os.path.join(dir, uuid[0:2], uuid[2:4], uuid[4:6],
                            uuid[6:8], uuid[8:10], uuid[10:12], uuid[12:14],
                            uuid[14:16], uuid[16:])
        exists = os.path.exists(path)
        if exists:
            print "UUID %s is not unique (run %s)" % (uuid, run)
            print "ran for %s seconds" % (time.time() - start)
            break
        os.makedirs(path)
        run = run + 1
